//
// bstring.h
//
//	String class for Basic to C++ translator
//
// Description:
//
//	This file impliments a string class for my Basic
//	translater using VMS style descriptors and string
//	functions. This allows the programs to interface
//	with the VMS routines as easily as under the
//	real VAX/BASIC compiler.
//
//	This is really nothing more than a wrapper around
//	the STR$xxx functions, and the DSC$DESCRIPTOR_S
//	stuff.
//
// History:
//
//	01.00.00 - 09/19/1996 - Kevin Handy
//		Original version. Very minimal. Assignment and output
//		only exist. All code contained in '.h' file (easier
//		to debug).
//
//	01.02.01 - 09/20/1996 - Kevin Handy
//		Added in '+' operator for BString and char.
//
//	01.02.02 - 01/01/1997 - Kevin Handy
//		Added sever string functions, including left,
//		right, pos, mid, string, space, strlen.
//
//	01.02.03 - 01/04/1997 - Kevin Handy
//		Added comparison operators (Bstring to Bstring)
//
//	01.02.04 - 01/05/1997 - Kevin Handy
//		Added more functions. Qnum, Qnum1
//

#ifndef _bstring_h_
#define _bstring_h_

//
// Include files
//
extern "C"
{
#include <stdio.h>
#include "descrip.h"
#include "strdef.h"
}
#include <iostream.h>

/*
 * Define some simple types that vary from machine to machine
 */
typedef short int intx;		// Default integer type
typedef double floatx;		// Default float type

//
// Hard coded constants
//
static const char BEL = 7;		// Bell
static const char BS = 8;		// Backspace
static const char HT = 9;		// Horizontal tab
static const char LF = 10;		// Line feed
static const char VT = 11;		// Vertical tab
static const char FF = 12;		// Form feed
static const char CR = 13;		// Carriage return
static const char SO = 14;		// Shift out
static const char SI = 15;		// Shift in
static const char ESC = 27;		// Escape
static const char SP = 32;		// Space
static const char DEL = 127;		// Delete
//static const double PI = 3.1415926535;// Pi (Someone else already defined this)

//
// BString class
//
class BString : public dsc$descriptor_s
{
public:
	BString();
	BString(const char* Text);
	~BString();

	//
	// Operators
	//
	BString& operator = (const char* Text);
	BString& operator = (const BString& Source)
		{ str$copy_dx(this, &Source); return *this; }

	//
	// Functions
	//

	//
	// I/O functions (Hack, needs lots of work)
	//
	friend ostream& operator<< (ostream& os, BString& bs)
		{ if (bs.dsc$a_pointer != NULL) {for (int loop=0; loop<bs.dsc$w_length; loop++)
			os << bs.dsc$a_pointer[loop];} return os; }
	friend istream& operator>> (istream& is, BString& bs)
		{ char inputstr[128]; is >> inputstr; bs = inputstr; }

	//
	// Operators
	//
	friend BString& operator + (const BString& Text1, const BString& Text2);
	friend BString& operator + (const BString& Text1, const char* Text2);
	friend int operator == (const BString& Text1, const BString& Text2);
	friend int operator < (const BString& Text1, const BString& Text2);
	friend int operator <= (const BString& Text1, const BString& Text2);
	friend int operator > (const BString& Text1, const BString& Text2);
	friend int operator >= (const BString& Text1, const BString& Text2);
	friend int operator != (const BString& Text1, const BString& Text2);
};


//
// Constructor
//
BString::BString()
{
// cout << "MakeD" << endl;
	dsc$b_dtype = DSC$K_DTYPE_T;		// Text class
	dsc$b_class = DSC$K_CLASS_D;		// Dynamic
	dsc$w_length = 0;			// Length = 0
	dsc$a_pointer = NULL;			// No allocation
}

BString::BString(const char* Text)
{
// cout << "MakeS" << endl;
	dsc$b_dtype = DSC$K_DTYPE_T;		// Text class
	dsc$b_class = DSC$K_CLASS_S;		// Static
	dsc$w_length = strlen(Text);		// Length
	dsc$a_pointer = (char*)Text;		// Point at origional string
}

//
// Destructor
//
BString::~BString()
{
// cout << "Killed" << endl;
	//
	// Release memory allocation if this is a string class
	//
	if ((dsc$b_class == DSC$K_CLASS_D) && (dsc$a_pointer != NULL))
	{
		str$free1_dx(this);
	}
}

//
// Assignment
//
BString& BString::operator = (const char* Text)
{
	short Length = strlen(Text);

	str$copy_r(this, &Length, Text);
	return *this;
}

//
// Left()
//
inline BString& left(const BString& Source, long Length)
{
	BString* NewString = new BString;
	str$left(NewString, &Source, &Length);
	return *NewString;
}

//
// Right()
//
inline BString& right(const BString& Source, long Length)
{
	BString* NewString = new BString;
	str$right(NewString, &Source, &Length);
	return *NewString;
}

//
// pos()
//
inline BString& pos(const BString& Source, long Start, long End)
{
	BString* NewString = new BString;
	str$pos_extr(NewString, &Source, &Start, &End);
	return *NewString;
}

//
// mid()
//
inline BString& mid(const BString& Source, long Start, long Length)
{
	BString* NewString = new BString;
	long End = Start + Length - 1;
	str$pos_extr(NewString, &Source, &Start, &End);
	return *NewString;
}

//
// string
//
inline BString& Qstring(long Count, int Character)
{
	BString* NewString = new BString;
	char Source = Character;

	str$dupl_char(NewString, &Count, &Source);

	return *NewString;
}

//
// space
//
inline BString& Qspace(long Count)
{
	BString* NewString = new BString;
	char XSpace = ' ';

	str$dupl_char(NewString, &Count, &XSpace);

	 return *NewString;
}

//
// Qnum(integer)
//
inline BString& Qnum(long Number)
{
	char Result[32];
	BString* NewString = new BString;

	sprintf(Result, "%ld ", Number);
	*NewString = Result;

	return *NewString;
}

//
// Qnum(float)
//
inline BString& Qnum(double Number)
{
	char Result[32];
	BString* NewString = new BString;

	sprintf(Result, "%f ", Number);
	*NewString = Result;

	return *NewString;
}

//
// Qnum1(integer)
//
inline BString& Qnum1(long Number)
{
	char Result[32];
	BString* NewString = new BString;

	sprintf(Result, "%ld", Number);
	*NewString = Result;

	return *NewString;
}

//
// Qnum1(float)
//
inline BString& Qnum1(double Number)
{
	char Result[64];
	BString* NewString = new BString;

	sprintf(Result, "%f", Number);
	*NewString = Result;

	return *NewString;
}

//
// trm
//
inline BString trm(const BString& Source)
{
	BString* NewString = new BString;
	unsigned short Length;

	str$trim(NewString, &Source, &Length);
	return *NewString;
}


//
// strlen
//
inline long strlen(const BString& Source)
{
	unsigned short Length;
	char* Address;

	str$analyze_sdesc(&Source, &Length, &Address);
	return Length;
}


//
// Addition
//
BString& operator + (const BString& Text1, const BString& Text2)
{
	//
	// Allocate a new string
	//
	BString* NewString = new BString;

	//
	// Combine strings together.
	//	Using copy,aappend instead of concat, because varargs
	//	won't give me information about number of paramaters.
	//
	str$copy_dx(NewString, &Text1);
	str$append(NewString, &Text2);

	//
	// Give back the new string
	//
	return *NewString;
}

BString& operator + (const BString& Text1, const char* Text2)
{
	//
	// Allocate a new string
	//
	BString* NewString = new BString;
	BString Transform(Text2);

	//
	// Combine strings together
	//
	str$copy_dx(NewString, &Text1);
	str$append(NewString, &Transform);

	//
	// Give back the string
	//
	return *NewString;
}

int operator == (const BString& Text1, const BString& Text2)
{
	return !str$compare_eql(&Text1, &Text2);
}

int operator < (const BString& Text1, const BString& Text2)
{
	 return str$compare(&Text1, &Text2) == -1;
}

int operator <= (const BString& Text1, const BString& Text2)
{
	return str$compare(&Text1, &Text2) <= 0;
}

int operator > (const BString& Text1, const BString& Text2)
{
	return str$compare(&Text1, &Text2) == 1;
}

int operator >= (const BString& Text1, const BString& Text2)
{
	return str$compare(&Text1, &Text2) >= 0;
}

int operator != (const BString& Text1, const BString& Text2)
{
	return str$compare(&Text1, &Text2) != 0;
}

//
// Various string type functions
//
static inline int Ascii(const char x) { return x; }
static inline int Ascii(const char* x) { return *x; }

#endif

