/*******************************************************************************

Program:

	variable.cc
	01.00.00

Title:

	Routines to handle variables.

Description:

	This file contains routines to handle the variable definitions
	necessary for basic.

Bugs:

History:

	01.01.00 - 08/30/1996 - Kevin Handy
		Update version number to 1.1, and update on internet.

	01.01.01 - 10/22/1996 - Kevin Handy
		Handle new 'IntegerType' and 'RealType' variables.

*******************************************************************************/

/*
 * System Include Files
 */
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>

//
// Project Include Files
//
#include "variable.h"
#include "basic.h"
#include "parse.h"

//
// Module #defined Constants
//

//
// Module #defined Functions
//

//
// Module Structure Definitions
//

//
// Module Function Prototypes
//

//
// Module Variables
//


/*******************************************************************************

Member Function:

	VariableStruct()

Title:

	Constructs a variable item.

Description:

	This routine fill add a variable to the variable table.

Bugs:

*******************************************************************************/

VariableStruct::VariableStruct
(
	char *VarName,
	int newType,
	int newClass,
	int newOutput
)
{
	BasicName = NULL;
	Type = newType;
	Class = newClass;
	CName = NULL;
	Output = newOutput;
	Left = NULL;
	Right = NULL;
//	Array = NULL;
	EverUsed = 0;

	//
	// Allocate space for variable name
	//
	if (VarName)
	{
		char* ThisName = new char[strlen(VarName)+1];
		strcpy(ThisName, VarName);
		BasicName = ThisName;
	}

}

/*******************************************************************************

Member Function:

	~VariableStruct()

Title:

	Destruct a variable.

Description:

	Destroys variables.

Bugs:

*******************************************************************************/

VariableStruct::~VariableStruct()
{
	delete BasicName;
	delete CName;
}

/*******************************************************************************

Member Function:

	Dump()

Title:

	Print out variable.

Description:

	Prints out a variables information.

Bugs:

	Ugly output.

*******************************************************************************/

void VariableStruct::Dump()
{
	//
	// Print out this variable
	//
	cout << "Var: " << BasicName;

	if (Class == VARCLASS_ARRAY)
	{
		cout << "[]";
	}

	cout << ", Type " << Type;

	switch (Type)
	{
	case VARTYPE_DYNSTR:
		cout << " Dynamic String";
		break;

	case VARTYPE_FIXSTR:
		cout << " Fixed length string";
		break;

	case VARTYPE_WORD:
		cout << " 16 bit integer";
		break;

	case VARTYPE_INTEGER:
		cout << " integer (" << IntegerType << ")";
		break;

	case VARTYPE_LONG:
		cout << " 32 bit integer";
		break;

	case VARTYPE_REAL:
		cout << " float(" << RealType << ")";
		break;

	case VARTYPE_DOUBLE:
		cout << " double";
		break;

	case VARTYPE_STRUCT:
		cout << " Structure";
		break;

	case VARTYPE_VOID:
		cout << " Void";
		break;

	case VARTYPE_BYTE:
		cout << " Byte";
		break;

	case VARTYPE_RFA:
		cout << " RFA";
		break;

	case VARTYPE_LABEL:
		cout << " Label";
		break;

	case VARTYPE_DECIMAL:
		cout << " Decimal";
		break;

	case VARTYPE_SINGLE:
		cout << " Single";
		break;

	case VARTYPE_GFLOAT:
		cout << " GFloat";
		break;

	case VARTYPE_HFLOAT:
		cout << " HFloat";
		break;

	default:
		cout << " Undefined";
		break;
	}

	cout << ", Class " << Class;

	switch (Class)
	{
	case VARCLASS_NONE:
		cout << " Regular variable";
		break;

	case VARCLASS_ARRAY:
		cout << " An Array";
		break;

	case VARCLASS_FUNC:
		cout << " Function";
		break;

	case VARCLASS_CONST:
		cout << " Constant";
		break;

	default:
		cout << " Undefined";
		break;
	}

	if (CName != 0)
	{
		cout << ", CName " << CName;

		if (Class == VARCLASS_ARRAY)
		{
			cout << "[]";
		}
	}

	cout << endl;
}


/*******************************************************************************

Member Function:

	VariableStruct::OutputDef()

Title:

	Dump out one variable node.

Description:

	This function will output one variables definition to the
	output file. These are only used if the definition isn't output
	as part of the source code.

Bugs:


*******************************************************************************/

void VariableStruct::OutputDef
(
	ostream& os,
	int Level
)
{
	//
	// Local Variables
	//
	char* OldSep;
	int ActualType = Type;

	//
	// Skip if already output
	//
	if (Output == TRUE)
	{
		return;
	}

	//
	// Tab over
	//
	Indent(os, Level);

	//
	// Validation Check
	//
	assert(BasicName != 0);

	//
	// Output This Variable
	//
	switch (Class)
	{
	case VARCLASS_CONST:
		os << "const ";
		break;
	}

	//
	// Change "Integer" and "Real" to more exact type
	//
	switch(ActualType)
	{
	case VARTYPE_INTEGER:
		ActualType = IntegerType;
		break;

	case VARTYPE_REAL:
		ActualType = RealType;
		break;
	}

	switch(ActualType)
	{
	case VARTYPE_DYNSTR:
		os << "BString ";
		break;

	case VARTYPE_FIXSTR:
		os << "char ";
		break;

	case VARTYPE_INTEGER:
		os << "intx ";
		break;

	case VARTYPE_WORD:
		os << "short int ";
		break;

	case VARTYPE_LONG:
		os << "long ";
		break;

	case VARTYPE_REAL:
		os << "floatx ";
		break;

	case VARTYPE_DOUBLE:
		os << "double ";
		break;

	case VARTYPE_VOID:
		os << "void ";
		break;

	case VARTYPE_BYTE:
		os << "char ";
		break;

	case VARTYPE_RFA:
		os << "RFA ";
		break;

	case VARTYPE_DECIMAL:
		os << "Decimal ";
		break;

	case VARTYPE_SINGLE:
		os << "float ";
		break;

	case VARTYPE_GFLOAT:
		os << "double ";
		break;

	case VARTYPE_HFLOAT:
		os << "long double ";
		break;

	default:
		os << "unknown ";
		break;

	}

	//
	// Pick a good name
	//
	if (CName != 0)
	{
		os << CName;
	}
	else
	{
		os << BasicName;
	}

	switch(Type)
	{
	case VARTYPE_FIXSTR:
		os << "[]";
		break;
	}


	//
	// Add brackets if this is an array definition
	//
	switch (Class)
	{
	case VARCLASS_ARRAY:
		//
		// Default array dimension under Basic is 0..10
		// for most Basic's I've seen.
		//
		os << "[11]";
		break;

	case VARCLASS_FUNC:
		os << "()";
		break;
	}

	os << ";" << endl;

	//
	// Flag this one as having been output.
	//
	Output = TRUE;
}

/*******************************************************************************

Local Function:

	Indent()

Title:

	Indents to current level.

Description:

	This function is used to indent the code.

Bugs:


*******************************************************************************/

void VariableStruct::Indent(ostream& os, int Level)
{
	int TabLoop;

	for (TabLoop = 1; TabLoop <= Level; TabLoop++)
	{
		os << "\t";
	}
}

/*******************************************************************************

Member Function:

	SetCName()

Title:

	Sets up the C Variable Name.

Description:

	Assigned the C variable name.

Bugs:


*******************************************************************************/

void VariableStruct::SetCName(const char* NewName)
{
	//
	// Lose old value
	//
	delete CName;

	//
	// Move over new value
	//
	CName = new char[strlen(NewName) + 1];
	strcpy(CName, NewName);
}

