/*******************************************************************************

Program:

	Variable.h
	01.01

Title:

	Header file for variable table

Description:

	This is the header file for the variable table/

Bugs:

	Lousy. Needs complete re-write

History:

	01.01.02 - 10/22/1996 - Kevin Handy
		Added 'SINGLE', "GFLOAT', HFLOAT' types.

*******************************************************************************/

//
// Variable.h - Some stuff just in case if isn't defined anywhere else
//
#ifndef _VARIABLE_H_
#define _VARIABLE_H_

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

//
// Include files
//
#include <iostream.h>
#include "nodes.h"

/*
 * Variable Type Definitions
 */
#define VARTYPE_DYNSTR	1	/* Dynamic String */
#define VARTYPE_FIXSTR	2	/* Fixed length string */
#define VARTYPE_WORD	3	/* 16 bit integer */
#define VARTYPE_INTEGER	4	/* 16 bit integer */
#define VARTYPE_LONG	5	/* 32 bit integer */
#define VARTYPE_REAL	6	/* float type */
#define VARTYPE_DOUBLE	7	/* double type */
#define VARTYPE_STRUCT	8	/* Structure */
#define VARTYPE_VOID	9	/* Void (no) return */
#define VARTYPE_BYTE	10	/* Byte (char) */
#define VARTYPE_RFA	11	/* RFA */
#define VARTYPE_LABEL	12	/* Label */
#define VARTYPE_DECIMAL	13	/* Label */
#define VARTYPE_SINGLE	14	/* Single precision float */
#define VARTYPE_GFLOAT	15	/* GFloat floating point */
#define VARTYPE_HFLOAT	16	/* HFloat floating point */

#define VARCLASS_NONE	0	/* Regular variable */
#define VARCLASS_ARRAY	1	/* An Array */
#define VARCLASS_FUNC	2	/* Function */
#define VARCLASS_CONST	3	/* Constant */
#define VARCLASS_LABEL	4	/* Label */


//
// Structure defining variables
//
class VariableStruct
{
public:
	char *BasicName;		/* Name used in BASIC */
	int Type;			/* Type (integer, real, ...) */
	int Class;			/* Variable Class (function, array, ...) */
	char *CName;			/* Name to be used in C */
	int Output;			/* Already output to source? */
	struct VariableStruct *Left;	/* Pointer to next variable (Less) */
	struct VariableStruct *Right;	/* Pointer to next variable (Greater) */
//	Node *Array;		/* Array Definition Info */
	int EverUsed;			/* Is it ever used (Label) */

public:
	VariableStruct(char *VarName = "", int Type = 0,
		int Class = 0, int Output = FALSE);
	~VariableStruct();

	VariableStruct *VariableAdd();
	void Dump();
	void OutputDef(ostream& os, int Level);
	char* GetName() { if (CName != "") return CName; else return BasicName; }
	void SetCName(const char* NewName);
	void SetClass(const int NewClass) { Class = NewClass; }

private:
	void Indent(ostream& os, int Level);
};

//
// Class that holds a list of variables
class VariableList
{
private:
	VariableStruct* Global;
	VariableStruct* Local;

public:
	VariableList();
	~VariableList();

	void VariableList::InitFunctions(void);
	void InitOneFunction(char *BasName, char *CName, int Type);

	VariableList* Append(VariableStruct* Variable, int GlobalFlag = FALSE);
	VariableStruct *Lookup(char *VarName, Node *ArrayRef);
	VariableStruct *LookupCpp(char *VarName);
	int LookupReserved(char *VarName);
	void Define(Node *ThisNode, int GlobalFlag = FALSE,
		int ThisOutput = FALSE);
	void Dump();
	void Fixup();
	void OutputDef(ostream& os, int Level);
	void ClearLocal();

private:
	void Fixup1(VariableStruct* Variable);
	void Fixup2(VariableStruct* Variable);
	void Fixup3(VariableStruct* Variable);
	VariableStruct *LookupCppTree(VariableStruct* Variable, char *VarName);
	void VariableList::DumpTree(VariableStruct* Variable);
	void VariableList::DeleteTree(VariableStruct* Node);
	void VariableList::OutputDefTree(ostream& os,
		VariableStruct* VarTable, int Level);
};

#endif
