/*******************************************************************************

Program:

	varlist.cc
	01.00.00

Title:

	Routines to handle variables.

Description:

	This file contains routines to handle the variable definitions
	necessary for basic.

Bugs:

History:

	01.01.00 - 08/30/1996 - Kevin Handy
		Update version to 1.1, and install on internet.

	01.01.01 - 10/09/1996 - Kevin Handy
		Changes "ascii" from "(char)(x)" to "Ascii(x)".

	01.01.02 - 10/20/1996 - Kevin Handy
		Output 'sqrt' instead of 'sqr' for square root.
		Added several more functions to table.

	01.02.01 - 01/01/1997 - Kevin Handy
		Output Qstring when STRING$() is callewd, because of
		problems with existing c++ functions.
		Also Qspace for space.

	01.02.02 - 01/02/1997 - Kevin Handy
		Output "trm" instead of "trim" for the trim function

	01.02.03 - 01/05/1997 - Kevin Handy
		Handle num$ and num1$ as Qnum and Qnum1.

*******************************************************************************/

/*
 * System Include Files
 */
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>

//
// Project Include Files
//
#include "variable.h"
#include "basic.h"
#include "parse.h"

//
// Module #defined Constants
//

//
// Module #defined Functions
//

//
// Module Structure Definitions
//

//
// Module Function Prototypes
//

//
// Module Variables
//


/*******************************************************************************

Member Function:

	VariableList()

Title:

	Constructs a variable list.

Description:

	This routine creates a variable list.

Bugs:

*******************************************************************************/

VariableList::VariableList()
{
	Global = NULL;
	Local = NULL;

	//
	// Load in hard coded function list
	//
	InitFunctions();
}

/*******************************************************************************

Member Function:

	~VariableList()

Title:

	Destruct a variable List.

Description:

	Destroys variable Lists.

Bugs:

*******************************************************************************/

VariableList::~VariableList()
{

	DeleteTree(Local);
	DeleteTree(Global);
}

void VariableList::DeleteTree(VariableStruct* Nodex)
{
	if (Nodex != NULL)
	{
		if (Nodex->Left != NULL)
		{
			DeleteTree(Nodex->Left);
		}

		if (Nodex->Right != NULL)
		{
			DeleteTree(Nodex->Right);
		}

		delete Nodex;
	}
}

/*******************************************************************************

Member Function:

	VariableList::Lookup()

Title:

	Look for a variable in the table using it's basic name.

Description:

	This routine will scan the variable table for the occurance of a
	specified variable.

Bugs:

*******************************************************************************/

VariableStruct *VariableList::Lookup
(
	char *VarName,
	Node *ArrayRef
)
{
	VariableStruct *VarTable = Local;

	//
	// Scan the list
	//
	while (VarTable != NULL)
	{
		int Direction;

		//
		// Is this it?
		//
		Direction = strcmp(VarTable->BasicName, VarName);
		if (Direction == 0)
		{
			//
			// Handle allowing a variable name as being
			// an array and a non-array in the same
			// program.
			//
			switch(VarTable->Class)
			{
			case VARCLASS_ARRAY:
				if (ArrayRef != 0)
				{
					return(VarTable);
				}
				VarTable = VarTable->Left;
				break;

			case VARCLASS_FUNC:
				return(VarTable);

			default:
				if (ArrayRef == 0)
				{
					return(VarTable);
				}
				VarTable = VarTable->Left;
				break;
			}
		}
		else
		{
			if (Direction > 0)
			{
				VarTable = VarTable->Right;
			}
			else
			{
				VarTable = VarTable->Left;
			}
		}
	}

	VarTable = Global;

	//
	// Scan the list
	//
	while (VarTable)
	{
		int Direction;

		//
		// Is this it?
		//
		Direction = strcmp(VarTable->BasicName, VarName);
		if (Direction == 0)
		{
			//
			// Handle allowing a variable name as being
			// an array and a non-array in the same
			// program.
			//
			switch(VarTable->Class)
			{
			case VARCLASS_ARRAY:
				if (ArrayRef != 0)
				{
					return(VarTable);
				}
				VarTable = VarTable->Left;
				break;

			case VARCLASS_FUNC:
				return(VarTable);

			default:
				if (ArrayRef == 0)
				{
					return(VarTable);
				}
				VarTable = VarTable->Left;
				break;
			}
			break;
		}
		else
		{
			if (Direction > 0)
			{
				VarTable = VarTable->Right;
			}
			else
			{
				VarTable = VarTable->Left;
			}
		}
	}

	//
	// Didn't find it
	//
	return(0);
}

/*******************************************************************************

Mamber Function:

	VariableList::LookupCpp()

Title:

	Look for a variable in the table using the C++ name.

Description:

	This routine will scan the variable table for the occurance of a
	specified variable.

Bugs:

*******************************************************************************/

VariableStruct *VariableList::LookupCpp
(
	char *VarName
)
{
	VariableStruct *VarTable = NULL;

	if (Local != NULL)
	{
		VarTable = LookupCppTree(Local, VarName);
	}

	if ((Global != NULL) && (VarTable == NULL))
	{
		VarTable = LookupCppTree(Global, VarName);
	}

	return VarTable;
}

VariableStruct *VariableList::LookupCppTree
(
	VariableStruct* Nodex,
	char* VarName
)
{
	VariableStruct *VarTable;

	//
	// Is this it?
	//
	if ((Nodex->CName != NULL) &&
		(strcmp(Nodex->CName, VarName) == 0))
	{
		return(Nodex);
	}

	//
	// Look at left side of nodex
	//
	if (Nodex->Left != NULL)
	{
		VarTable = LookupCppTree(Nodex->Left, VarName);
		if (VarTable != NULL)
		{
			return VarTable;
		}
	}

	//
	// Look at right side of nodex
	//
	if (Nodex->Right != NULL)
	{
		VarTable = LookupCppTree(Nodex->Right, VarName);
		if (VarTable != NULL)
		{
			return VarTable;
		}
	}


	//
	// Didn't find it
	//
	return NULL;
}

/*******************************************************************************

Mamber Function:

	VariableList::LookupReserved()

Title:

	Look for a variable in the list of reserved names and the
	already used names using the C++ name.

Description:

	This routine will scan the variable table for the occurance of a
	specified variable.

Bugs:

*******************************************************************************/

//
// This list represent names that are not available for one reason or
// another to a C++ program for use as a variable name. It keeps
// generated names from overlapping too badly with existing functions
// and reserved words.
//
// Do not include constants that have been appended to the variable
// list in the startup routines. That would be redundant, and a
// duplication of effort.
//
static const char* ReservedList[] =
{
	"atof",
	"atoi",
	"auto",
	"break",
	"case",
	"char",
	"cin",
	"class",
	"const",
	"continue",
	"cout",
	"default",
	"delete",
	"do",
	"double",
	"else",
	"extern",
	"far",
	"fclose",
	"float",
	"fopen",
	"for",
	"fprintf",
	"free",
	"goto",
	"if",
	"int",
	"long",
	"malloc",
	"namespace",
	"near",
	"new",
	"operator",
	"printf",
	"public",
	"private",
	"register",
	"return",
	"scanf",
	"short",
	"sprintf",
	"static",
	"strcmp",
	"struct",
	"switch",
	"union",
	"using",
	"virtual",
	"void",
	"volatile",
	"template",
	"throw",
	"typedef",
	"unsigned",
	"while",
	0
};

int VariableList::LookupReserved
(
	char *VarName
)
{
	//
	// Look through reserved word list
	//
	for (int i = 0; ReservedList[i] != 0; i++)
	{
		if (strcmp(ReservedList[i] , VarName) == 0)
		{
			return TRUE;
		}
	}

	//
	// Look through list of already used names
	//
	VariableStruct *VarTable = LookupCpp(VarName);

	if (VarTable != NULL)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/*******************************************************************************

Member Function:

	VariableList::Dump()

Title:

	Print out variable table.

Description:

	Prints out the variables stored in the variable table.

Bugs:

	Ugly output.

*******************************************************************************/

void VariableList::Dump()
{
	//
	// Dump out global variables
	//
	cerr << endl << "Global Variables:" << endl;
	if (Global != NULL)
	{
		DumpTree(Global);
	}

	//
	// Dump out local variables
	//
	cerr << endl << "Local Variables:" << endl;
	if (Local != NULL)
	{
		DumpTree(Local);
	}

	cerr << endl;
}

void VariableList::DumpTree(VariableStruct* Variable)
{
	if (Variable != NULL)
	{
		DumpTree(Variable->Right);

		Variable->Dump();

		DumpTree(Variable->Left);
	}
}


/*******************************************************************************

Member Function:

	VariableList::Define()

Title:

	Define the current node as a variable.

Description:

	This function tries to define the current node as a variable.
	It is used to define those variables that are not defined
	in some other manner by the user.

Bugs:

*******************************************************************************/

void VariableList::Define(Node *ThisNode, int GlobalFlag, int ThisOutput)
{
	//
	// Local Variables
	//
	int Type;
	VariableStruct *ThisVar;
	int ThisClass = VARCLASS_NONE;

	//
	// If the name starts fn..., then it must be a function
	// (it's the law)
	//
	if (strncmp(ThisNode->GetText(), "FN", 2) == 0)
	{
		ThisNode->SetType(BAS_V_FUNCTION);
	}

	//
	// Is it already defined as a variable?
	//
	if ((ThisVar = Lookup(ThisNode->GetText(),
		ThisNode->GetTree(0))) != NULL)
	{
		//
		// Force the node to be a function if that is what it is.
		//
		if (ThisVar->Class == VARCLASS_FUNC)
		{
			ThisNode->SetType(BAS_V_FUNCTION);
		}
		return;
	}

	//
	// Check the last letter of the variable to determine what type
	// it ought to be.
	//
	switch(ThisNode->GetText()[strlen(ThisNode->GetText()) - 1])
	{
	case '$':
		Type = VARTYPE_DYNSTR;
		break;

	case '%':
		Type = VARTYPE_INTEGER;
		break;

	default:
		Type = VARTYPE_REAL;
		break;
	}

	//
	// Is this an array reference?
	//
	if (ThisNode->GetTree(0) != NULL)
	{
		ThisClass = VARCLASS_ARRAY;
	}

	//
	// Add the variable to the list
	//
	VariableStruct* NewVariable = new VariableStruct(
		ThisNode->GetText(), Type, ThisClass, ThisOutput);
	Append(NewVariable, GlobalFlag);
}


/*******************************************************************************

Member Function:

	VariableList::ClearLocal()

Title:

	Destroy the local variable List.

Description:

	Destroys local variable Lists, leaving the global list alone.

Bugs:

*******************************************************************************/

void VariableList::ClearLocal()
{
	VariableStruct* ThisItem = Local;
	VariableStruct* NextItem;

	//
	// Delete everything saved up
	//
	DeleteTree(Local);

	//
	// Clear pointer
	//
	Local = 0;
}

/*******************************************************************************

Member Function:

	Append()

Title:

	Add a variable to the table.

Description:

	This routine will add a variable to the end of the variable table.

Bugs:

	Assumes variable is not already in the table.

*******************************************************************************/

VariableList *VariableList::Append(VariableStruct* Variable, int GlobalFlag)
{
	VariableStruct* Down;

	if (GlobalFlag)
	{
		Down = Global;
	}
	else
	{
		Down = Local;
	}

	//
	// Insert it into the table
	//
	if (Down != NULL)
	{
		//
		// Find a good place to attach it to
		//
		while(1)
		{
			int Compare = strcmp(Down->BasicName,
				Variable->BasicName);

			if (Compare > 0)
			{
				if (Down->Right != NULL)
				{
					Down = Down->Right;
				}
				else
				{
					Down->Right = Variable;
					return this;
				}
			}
			else
			{
				if (Down->Left != NULL)
				{
					Down = Down->Left;
				}
				else
				{
					Down->Left = Variable;
					return this;
				}
			}
		}
	}
	else
	{
		if (GlobalFlag)
		{
			Global = Variable;
		}
		else
		{
			Local = Variable;
		}
	}
	return(this);
}


/*******************************************************************************

Member Function:

	VariableList::Fixup()

Title:

	Fix up variable names

Description:

	This function changes all of the variable names from
	formats that BASIC likes, into those that C likes.

Algorithm:

	1. Convert those variables that can be to simply lower case
	   versions of the upper case basic names.

	2. For variables with trailing characters "%", try leaving
	   the "%" off, and make sure there are no duplicates.

	3. Any variables left over, convert to lower case, changing
	   unallowed characters to underscores. Make sure that they
	   don't duplicate any other name.

	4. Additional to step 3, add a "Vn" to the variable to make
	   unique.

Bugs:

*******************************************************************************/

void VariableList::Fixup()
{
	//
	// Local Variables
	//

	//
	// Do the various stages of fixup
	//
	Fixup1(Local);
	Fixup2(Local);
	Fixup3(Local);
}

/*******************************************************************************

Member Function:

	Fixup1()

Title:

	Fix up variable names

Description:

	1. Convert those variables that can be to simply lower case
	   versions of the upper case basic names.

Bugs:

*******************************************************************************/

void VariableList::Fixup1(VariableStruct* VarList)
{
	//
	// Local Variables
	//
	char tempname[64];
	char *ThisName;

	//
	// Do the various stages of fixup
	//
	if (VarList != NULL)
	{
		//
		// Don't re-convert, don't allow illegal basic characters.
		//
		if ((VarList->CName == NULL) &&
			(strchr(VarList->BasicName, '.') == 0) &&
			(strchr("%$", VarList->BasicName[
				strlen(VarList->BasicName)-1]) == 0))
		{
			if ((VarList->Type == VARTYPE_LABEL) &&
				(isdigit(VarList->BasicName[0])))
			{
				//
				// Lables are never reserved, so always
				// give them a specific value
				//
				strcpy(tempname, "L_");
				strcat(tempname, VarList->BasicName);
				ThisName = new char[strlen(tempname)+1];
				strcpy(ThisName, tempname);
				VarList->CName = ThisName;
			}
			else
			{
				strcpy(tempname, VarList->BasicName);
				strlwr(tempname);

				//
				// Don't assign if used by another variable
				//
				if (Variables->LookupReserved(tempname) ==
					FALSE)
				{
					ThisName = new char[strlen(tempname)+1];
					strcpy(ThisName, tempname);
					VarList->CName = ThisName;
				}
			}
		}

		//
		// Point to next
		//
		Fixup1(VarList->Left);
		Fixup1(VarList->Right);
	}
}

/*******************************************************************************

Member Function:

	Fixup2()

Title:

	Fix up variable names

Description:

	2. For variables with trailing characters "%", try leaving
		the "%" off, and make sure there are no duplicates.

Bugs:

*******************************************************************************/

void VariableList::Fixup2(VariableStruct* VarList)
{
	//
	// Local Variables
	//
	char tempname[64];
	char *ThisName;

	//
	// Do the various stages of fixup
	//
	if (VarList != NULL)
	{
		//
		// Don't re-convert, don't allow illegal basic characters.
		//
		if ((VarList->CName == NULL) &&
			(strchr(VarList->BasicName, '.') == 0) &&
			(strchr("%$", VarList->BasicName[
				strlen(VarList->BasicName) - 1]) != 0))
		{
			strcpy(tempname, VarList->BasicName);
			strlwr(tempname);

			//
			// Lose '$', '%'
			//
			tempname[strlen(tempname) - 1] = '\0';

			if (Variables->LookupReserved(tempname) == FALSE)
			{
				ThisName = new char[strlen(tempname)+1];
				strcpy(ThisName, tempname);
				VarList->CName = ThisName;
			}
		}

		//
		// Point to next
		//
		Fixup2(VarList->Left);
		Fixup2(VarList->Right);
	}
}

/*******************************************************************************

Member Function:

	VariableList::Fixup3()

Title:

	Fix up variable names

Description:

	3. Any variables left over, convert to lower case, changing
	   unallowed characters to underscores. Make sure that they
	   don't duplicate any other name.

	4. Additional to step 3, add a "Vn" to the variable to make
	   unique.

Bugs:

*******************************************************************************/

static int vcount = 0;		/* Current forced variable counter */

void VariableList::Fixup3(VariableStruct* VarList)
{
	//
	// Local Variables
	//
	char tempname[64];
	char number[6];
	char *ThisName;
	int loop;

	//
	// Do the various stages of fixup
	//
	if (VarList != NULL)
	{
		//
		// Don't re-convert, don't allow illegal basic characters.
		//
		if (VarList->CName == NULL)
		{
			if ((VarList->Type == VARTYPE_LABEL) &&
				(isdigit(VarList->BasicName[0])))
			{
				strcpy(tempname, "L_");
				strcat(tempname, VarList->BasicName);
			}
			else
			{
				strcpy(tempname, VarList->BasicName);
				strlwr(tempname);
			}

			if (strchr("%$", tempname[strlen(tempname)-1]) != 0)
			{
				tempname[strlen(tempname)-1] = '\0';
			}

			for (loop = 0; tempname[loop] != 0; loop++)
			{
				if (strchr(".", tempname[loop]) != 0)
				{
					tempname[loop] = '_';
				}
			}

			if (Variables->LookupReserved(tempname) == FALSE)
			{
				ThisName = new char[strlen(tempname)+1];
				strcpy(ThisName, tempname);
				VarList->CName = ThisName;
			}
			else
			{
				strcat(tempname, "_V");
				sprintf(number, "%d", vcount++);
				strcat(tempname, number);

				ThisName = new char[strlen(tempname) + 1];
				strcpy(ThisName, tempname);
				VarList->CName = ThisName;
			}
		}

		//
		// Point to next
		//
		Fixup3(VarList->Left);
		Fixup3(VarList->Right);
	}
}

/*******************************************************************************

Member Function:

	VariableList::OutputDef()

Title:

	Output any variables that have not yet been dumped.

Description:

	This function will output any variables that have not been dumped
	by any previous routines.

Bugs:


*******************************************************************************/

void VariableList::OutputDef(ostream& os, int Level)
{
	OutputDefTree(os, Local, Level);
}

void VariableList::OutputDefTree(ostream& os,
	VariableStruct* VarTable, int Level)
{
	//
	// Determine type of variable
	//
	if (VarTable != NULL)
	{
		//
		// Handle left hand side first
		//
		OutputDefTree(os, VarTable->Right, Level);

		//
		// Output this variable
		//
		VarTable->OutputDef(os, Level);

		//
		// Handle rightt hand side first
		//
		OutputDefTree(os, VarTable->Left, Level);

	}
}

/*******************************************************************************

Local Function:

	VariableList::InitFunctions()

Title:

	Initialize function names.

Description:

	This function initializes several built in basic functions
	that have compatible C functions (same parameters).

Bugs:


*******************************************************************************/

void VariableList::InitFunctions(void)
{
	InitOneFunction("ABS",		"fabs",		VARTYPE_DOUBLE);
	InitOneFunction("ABS%",		"abs",		VARTYPE_DOUBLE);
	InitOneFunction("ASCII",	"Ascii",	VARTYPE_LONG);
	InitOneFunction("ATN",		"atan",		VARTYPE_DOUBLE);
	InitOneFunction("COS",		"cos",		VARTYPE_DOUBLE);
	InitOneFunction("CVT$%",	"cvtai",	VARTYPE_LONG);
	InitOneFunction("CVT%$",	"cvtia",	VARTYPE_DYNSTR);
	InitOneFunction("CVT$F",	"cvtaf",	VARTYPE_DOUBLE);
	InitOneFunction("CVTF$",	"cvtfa",	VARTYPE_DYNSTR);
	InitOneFunction("CVT$$",	"cvtaa",	VARTYPE_DYNSTR);
	InitOneFunction("EDIT$",	"edit",		VARTYPE_DOUBLE);
	InitOneFunction("EXP",		"exp",		VARTYPE_DOUBLE);
	InitOneFunction("FIX",		"fix",		VARTYPE_DOUBLE);
	InitOneFunction("FORMAT$",	"format",	VARTYPE_DYNSTR);
	InitOneFunction("INT",		"floor",	VARTYPE_DOUBLE);
	InitOneFunction("LEFT",		"left",		VARTYPE_DYNSTR);
	InitOneFunction("LEFT$",	"left",		VARTYPE_DYNSTR);
	InitOneFunction("LEN",		"strlen",	VARTYPE_LONG);
	InitOneFunction("LOC",		"&",		VARTYPE_LONG);
	InitOneFunction("LOG",		"log",		VARTYPE_DOUBLE);
	InitOneFunction("LOG10",	"log10",	VARTYPE_DOUBLE);
	InitOneFunction("MID",		"mid",		VARTYPE_DYNSTR);
	InitOneFunction("MID$",		"mid",		VARTYPE_DYNSTR);
	InitOneFunction("MAG",		"fabs",		VARTYPE_DOUBLE);
	InitOneFunction("NUM$",		"Qnum",		VARTYPE_DYNSTR);
	InitOneFunction("NUM1$",	"Qnum1",	VARTYPE_DYNSTR);
	InitOneFunction("RIGHT",	"right",	VARTYPE_DYNSTR);
	InitOneFunction("RIGHT$",	"right",	VARTYPE_DYNSTR);
	InitOneFunction("RND",		"floatrand",	VARTYPE_DOUBLE);
	InitOneFunction("SIN",		"sin",		VARTYPE_DOUBLE);
	InitOneFunction("SPACE$",	"Qspace",	VARTYPE_DYNSTR);
	InitOneFunction("SQRT",		"sqrt",		VARTYPE_DOUBLE);
	InitOneFunction("SQR",		"sqr",		VARTYPE_DOUBLE);
	InitOneFunction("STRING$",	"Qstring",	VARTYPE_DYNSTR);
	InitOneFunction("SWAP%",	"swab",		VARTYPE_LONG);
	InitOneFunction("TAN",		"tan",		VARTYPE_DOUBLE);
	InitOneFunction("TRM$",		"trm",		VARTYPE_DYNSTR);
	InitOneFunction("VAL",		"atof",		VARTYPE_DOUBLE);
	InitOneFunction("VAL%",		"atoi",		VARTYPE_LONG);
}

/*******************************************************************************

Local Function:

	VariableList::InitOneFunction()

Title:

	Initialize function names.

Description:

	This function fills in remaining blanks for a function.

Bugs:
	This function is defined to be used be InitFunctions only.


*******************************************************************************/
void VariableList::InitOneFunction(char *BasName, char *CName, int Type)
{
	VariableStruct *ThisVar;
	char *B1Name;
	char *C1Name;

	//
	// Create malloc'd copies of names
	//
	B1Name = new char [strlen(BasName) + 1];
	strcpy(B1Name, BasName);

	C1Name = new char[strlen(CName) + 1];
	strcpy(C1Name, CName);

	//
	// Create area for function definition
	//
	ThisVar = new VariableStruct;

	//
	// Slap together a structure
	//
	ThisVar->BasicName = B1Name;
	ThisVar->Type = Type;
	ThisVar->Class = VARCLASS_FUNC;
	ThisVar->CName = C1Name;
	ThisVar->Output = 1;
//	ThisVar->Array = NULL;

	//
	// Hook into variable definition table
	//
	Append(ThisVar, TRUE);
}

